%{
  #include <stdio.h>
  #include <string.h>
  #include <stdlib.h>
  extern int yylex();
  extern int yyerror();

  typedef struct sym_entry{
    char const *name; /* Name of the symbol */
    float value; /* Value of symbol */
    struct sym_entry *next; /* Next node */
  } sym_entry;

  sym_entry *sym_table = NULL;

  /* Function used to get an entry of the symbol table */
  float sym_table_get(char const *name){
    for (sym_entry *p = sym_table; p; p = p->next)
      if(strcmp (p->name, name) == 0)
        return p->value;
    return -1.0;
  }

  /* Function used to push an entry in the symbol table */
  sym_entry *sym_table_put(char const *name, float value){
    if(sym_table_get(name) != -1.0)
      return NULL;
    sym_entry *new_node = (sym_entry *) malloc(sizeof(sym_entry));
    new_node->name = name;
    new_node->value = value;
    new_node->next = sym_table;
    sym_table = new_node;
    return new_node;
  }

  /* Function used to free the symbol table */
  void free_sym_table(){
    sym_entry *p = sym_table;
    sym_entry *n = NULL;
    while(p != NULL) {
      n = p->next;
      free((void *)p->name);
      free(p);
      p = n;
    }
  }
%}

%union {
  float real_value;
  int int_value;
  char *string;
}

%debug
%locations

%type <real_value>  purchasedProducts purchasedProduct
%type prog products product receipts receipt

%token <string> ID
%token <real_value> EURO
%token <int_value> INT
%token CM S C SEP

%%

prog : products SEP receipts;

products : products product | product;

product : ID EURO S { sym_table_put($1, $2); };

receipts : receipts receipt | ;  /* Empty list */

receipt : ID C purchasedProducts S
{
  printf("%s: %0.1f EURO\n", $1, $3);
  free((void *)$1);
};

purchasedProducts : purchasedProducts CM purchasedProduct { $$ = $1 + $3; }
    | purchasedProduct { $$ = $1; }
;

purchasedProduct : INT ID { $$ = $1 * sym_table_get($2); }
;
